@@ -9,7 +9,7 @@
 #'   whether to save the edited values in a csv file for later use.
 #' @param file Character. Path and name of custom language file to be saved.
 #'   This comma delimited file can be reused by calling
-#'   \code{\link{use_custom_lang}}.
+#'   \code{\link{use_custom_lang}}. Must have \emph{.csv} extension.
 #'
 #' @details On systems with GUI capabilities, a window will pop-up when calling 
 #' \code{define_keywords()} without any parameters, allowing the modification 
@@ -114,7 +114,7 @@
 #' @importFrom tcltk tclvalue tk_messageBox tkgetSaveFile
 #' @importFrom checkmate check_path_for_output
 #' @export
-define_keywords <- function(..., ask = FALSE, file = NA) {
+define_keywords <- function(..., ask = TRUE, file = NA) {
   mc <- match.call()
   kw <- names(mc[setdiff(names(mc), names(formals()))])[-1]
   if (length(kw) == 0 && !isTRUE(interactive())) {
@@ -130,6 +130,15 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
     return(invisible())
   }
   
+  # Validate file argument
+  if (!is.na(file)) {
+    filename <- normalizePath(file, mustWork = FALSE)
+    if (!isTRUE(check_path_for_output(filename, extension = "csv",
+                                      overwrite = TRUE))) {
+      stop("Invalid file name or extension (must be .csv)")
+    }
+  }
+
   if (st_options("lang") == "custom") {
     tr <- as.data.frame(t(.st_env$custom_lang), stringsAsFactors = FALSE)
   } else {
@@ -166,12 +175,13 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
     for (it in kw) {
       ind <- which(tr$item == it)
       if (length(ind) == 0) {
-        stop("'", it, "' is not a recognized keyword; see ?define_keywords ",
-             "for a list of valid keywords")
+        message("'", it, "' is not a recognized keyword; see ?define_keywords ",
+                "for a list of valid keywords")
+        next
       }
       if (inherits(mc[[it]], c("call", "name"))) {
        mc[[it]] <- eval(mc[[it]], parent.frame())
-      } 
+      }
       tr$custom[ind] <- mc[[it]]
     }
   }
@@ -179,31 +189,91 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
   use_custom_lang(tr)
 
   if (!is.na(file)) {
-    filename <- normalizePath(file, mustWork = FALSE)
-    if (isTRUE(check_path_for_output(filename, overwrite = TRUE))) {
     write.csv(x = tr,
               file = filename,
               row.names = FALSE, 
               fileEncoding = "utf-8")
     message("Custom language file written: ", filename)
-    } else {
-      warning("file name or path is invalid. Custom language is in effect; ",
-              "call define_keywords() without arguments to save file")
-    }
   } else if (isTRUE(ask)) {
     filename <- ""
     filename_ok <- FALSE
+    dialog_error <- NA
     
-    if (interactive() && length(kw) == 1) {
+    if (interactive()) { # && length(kw) == 1) {
       
-      if (isTRUE(capabilities("tcltk")) && isTRUE(ask)) {
+      dialog_error <- TRUE
+      filename_ok  <- FALSE
+      
+      # Get file name under RStudio (tcltk dialogs behave inconsistently)
+      if (.Platform$GUI == "RStudio") {
+        resp <- try(
+          rstudioapi::showQuestion(
+            title = "Keywords Successfully Updated",
+            message = "Export language file for later user?",
+            ok = "Yes", cancel = "No"
+          ),
+          silent = TRUE)
+        if (!inherits(resp, "try-error")) {
+          dialog_error <- FALSE
+          if (isTRUE(resp)) {
+            while (!filename_ok) {
+              filename <- rstudioapi::selectFile(
+                caption = "Save custom language file",
+                label = "Save",#Enter file name, including extension (.csv)",
+                path = getwd(),
+                filter = "CSV file (*.csv)",
+                existing = FALSE
+              )
+              if (is.null(filename)) {
+                # dialog cancelled -- validate cancel action
+                filename <- ""
+                filename_ok <- TRUE
+              } else {
+                # make sure csv / txt extension is there
+                if (!grepl("\\.(csv|txt)$", filename)) {
+                  rv <- rstudioapi::showQuestion(
+                    title = "Save custom language file",
+                    message = "Invalid file extension (must be .csv)",
+                    ok = "Retry"
+                  )
+                  if (rv == "cancel") {
+                    filename <- ""
+                    filename_ok <- TRUE
+                  } else {
+                    filename_ok <- FALSE
+                    next
+                  }
+                }
+                filename <- normalizePath(filename, mustWork = FALSE)
+                if (!isTRUE(check_path_for_output(filename, 
+                                                  overwrite = TRUE))) {
+                  rv <- rstudioapi::showQuestion(
+                    title = "Custom language file", 
+                    message = "Invalid file name or location",
+                    ok = "Retry",
+                    cancel = "Cancel"
+                  )
+                  if (!rv) {
+                    # cancel confirmed
+                    filename <- ""
+                    filename_ok <- TRUE
+                  }
+                } else {
+                  # Filename is valid
+                  filename_ok <- TRUE
+                }
+              } 
+            } # while
+          } # save yes
+        }
+      } else if (isTRUE(capabilities("tcltk"))) {
         # tcltk capabilities: yes
         resp <- try(tk_messageBox(type = "yesno", 
                                   message = "Export language file for later use?",
                                   caption = "Keywords Successfully Updated"),
                     silent = TRUE)
-        if (inherits(resp, "try-error")) {
-          tcltk_error <- TRUE
+        if (!inherits(resp, "try-error")) {
+          dialog_error <- FALSE
         } else {
           if (resp == "yes") {
             while (!filename_ok) {
@@ -218,8 +288,8 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
                 filename <- normalizePath(filename, mustWork = FALSE)
                 if (!isTRUE(check_path_for_output(filename, overwrite = TRUE))) {
                   rv <- tk_messageBox(
-                    type = "okcancel", 
-                    message = "Invalid file name or location"
+                    type = "retrycancel", 
+                    message = "Invalid file name or location."
                   )
                   if (rv == "cancel") {
                     filename <- ""
@@ -231,6 +301,7 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
                 }
               } else {
                 # dialog "Save as..." was cancelled
+                message("Custom language file not saved")
                 filename_ok <- TRUE
               }
             }
@@ -239,7 +310,7 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
       }
       
       # tcltk capabilities: no, or attempt failed
-      if (isFALSE(capabilities("tcltk")) || exists("tcltk_error")) {
+      if (isFALSE(capabilities("tcltk")) || isTRUE(dialog_error)) {
         resp <- " "
         while (!resp %in% c("Y", "N", "")) {
           resp <- toupper(
@@ -255,7 +326,7 @@ define_keywords <- function(..., ask = FALSE, file = NA) {
             if (filename != "" && 
                 (!isTRUE(check_path_for_output(filename, overwrite = TRUE)) || 
                  !grepl("\\.csv$", filename))) {
-              message("Invalid file location or extension (must be .csv)")
+              message("Invalid file name or extension (must be .csv)")
             } else {
               filename_ok <- TRUE # filename either "" or valid
             }
