@@ -96,7 +96,7 @@ parse_call <- function(mc,
   pos$expos   <- grep("^`%\\$%`\\(",sc_head)
   pos$lapply  <- grep("^lapply",    sc_head)
   pos$tapply  <- grep("^tapply",    sc_head)
-  pos$fun     <- grep(paste0("^", caller), sc_head)[1]
+  pos$fun     <- grep(paste0("^(summarytools::)?", caller), sc_head)[1]
   
   if (is.na(pos$fun)) {
     for (i in seq_along(sc)) {
@@ -147,17 +147,25 @@ parse_fun <- function()  {
   call <- standardize(.p$calls$fun)
   
   if (length(.p$var) > 1) {
-    #str <- paste0(deparse(.p$calls[[grep(.p$caller, .p$calls)[1]]]), collapse = "")
     done <- parse_data_str(deparse(call))
     return(done)
   }
   
-  obj <- try(.p$sf[[.p$pos$fun]][[.p$var]], silent = TRUE)
-  # Extract names from x argument
-  #nms <- all.names(call[[.p$var]])
-  #len <- length(call[[.p$var]])
+  # case of get(...)  
+  if (grepl("^get\\(", deparse(call[[.p$var]]))) {
+    obj_ <- try(eval(call[[.p$var]]), silent = TRUE)
+    if (is.data.frame(obj_)) {
+      df_name <- dynGet(x = deparse(call[[.p$var]][[2]]),
+                        ifnotfound = NULL, inherits = TRUE)
+      if (is.character(df_name) && length(df_name) == 1L) { 
+        done <- upd_output("df_name",  df_name)
+        done <- upd_output("df_label", label(obj_) %||% NA_character_)
+        if (done)  return(TRUE)
+      }
+    }
+  }
   
-  # experimental, 2025-02
+  # Identify names in the fn call
   nms <- unique(c(all.names(call[[.p$var]]), as.character(call[[.p$var]])))
   nms <- setdiff(
     nms,
@@ -166,6 +174,8 @@ parse_fun <- function()  {
   nms <- grep(pattern = "::", x = nms, fixed = TRUE, invert = TRUE, value = TRUE)
   len <- length(nms)
   
+  obj <- try(.p$sf[[.p$pos$fun]][[.p$var]], silent = TRUE)
+  
   if (is.data.frame(obj)) {
     if (len == 1) {
       if (identical(call[[.p$var]], as.name("."))) {
